/*===================== begin_copyright_notice ==================================

Copyright (c) 2017 Intel Corporation

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


======================= end_copyright_notice ==================================*/

#ifndef __SPLITALIGNEDSCALARS_H__
#define __SPLITALIGNEDSCALARS_H__

#include "visa/GraphColor.h"

namespace vISA
{
    class SplitAlignedScalars
    {
    private:
        const unsigned int MinOptDist = 200;
        unsigned int numDclsReplaced = 0;
        GlobalRA& gra;
        G4_Kernel& kernel;

        class Data
        {
        public:
            unsigned int firstDef = 0;
            unsigned int lastUse = 0;
            unsigned int numDefs = 0;
            unsigned int numSrcs = 0;
            unsigned int getDUMaxDist()
            {
                return std::abs((int)lastUse - (int)firstDef);
            };
            bool isSendDst = false;
        };

        std::unordered_map<G4_Declare*, Data> dclData;
        std::unordered_map<G4_Declare*, G4_Declare*> oldNewDcls;

        bool static canReplaceDst(G4_INST* inst);
        bool static canReplaceSrc(G4_INST* inst, unsigned int idx);

        bool heuristic(G4_Declare* dcl, Data& d);
        bool isDclCandidate(G4_Declare* dcl);
        void gatherCandidates();

        template<class T>
        G4_Declare* getDclForRgn(T* rgn, G4_Declare* newTopDcl);

    public:
        SplitAlignedScalars(GlobalRA& g) : gra(g), kernel(g.kernel)
        {
        }

        void run();

        void dump(std::ostream& of = std::cerr);
    };

}
#endif
